#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "Nebula_Device_AV.h"
#include "speaker.h"
#include <windows.h>

extern AV_Client gClientInfo[MAX_CLIENT_NUMBER];

static void usleep(long long usec) {
    HANDLE timer;
    LARGE_INTEGER interval;
    interval.QuadPart = -(10 * usec);

    timer = CreateWaitableTimer(NULL, TRUE, NULL);
    SetWaitableTimer(timer, &interval, 0, NULL, NULL, 0);
    WaitForSingleObject(timer, INFINITE);
    CloseHandle(timer);
}

static int gettimeofday(struct timeval *t, void *tz) {
    time_t _clock = 0;
    struct tm _tm = {0};
    SYSTEMTIME win_tm = {0};
    GetLocalTime(&win_tm);
    _tm.tm_year = win_tm.wYear - 1900;
    _tm.tm_mon = win_tm.wMonth - 1;
    _tm.tm_mday = win_tm.wDay;
    _tm.tm_hour = win_tm.wHour;
    _tm.tm_min = win_tm.wMinute;
    _tm.tm_sec = win_tm.wSecond;
    _tm.tm_isdst = -1;
    _clock = mktime(&_tm);
    t->tv_sec = _clock;
    t->tv_usec = win_tm.wMilliseconds * 1000;
    return 0;
}

//########################################################
//#Thread - Receive audio
//########################################################
static void WINAPI ThreadReceiveAudio(void *arg)
{
    int sid = *((int *)arg);
    free(arg);
    AV_Client *p = &gClientInfo[sid];
    int av_index = p->av_index;

    printf("\n[%s] start SID[%d] avIndex[%d]\n", __func__, sid, av_index);

    if(av_index > -1)
    {
        char buf[AUDIO_BUF_SIZE];
        FRAMEINFO_t frame_info;
        unsigned int frm_no = 0;
        int fps_count = 0;

        struct timeval tv1, tv2;
        gettimeofday(&tv1, NULL);
        while(p->enable_speaker) {
            int ret = avRecvAudioData(av_index, buf, AUDIO_BUF_SIZE, (char *)&frame_info, sizeof(FRAMEINFO_t), &frm_no);
            //printf("avRecvAudioData[%d]\n", ret);
            if(ret == AV_ER_SESSION_CLOSE_BY_REMOTE) {
                printf("[%s] AV_ER_SESSION_CLOSE_BY_REMOTE SID[%d] avIndex[%d]\n", __func__, sid, av_index);
                break;
            } else if(ret == AV_ER_REMOTE_TIMEOUT_DISCONNECT) {
                printf("[%s] AV_ER_REMOTE_TIMEOUT_DISCONNECT SID[%d] avIndex[%d]\n", __func__, sid, av_index);
                break;
            } else if(ret == IOTC_ER_INVALID_SID) {
                printf("[%s] Session cant be used anymore SID[%d] avIndex[%d]\n", __func__, sid, av_index);
                break;
            } else if(ret == AV_ER_LOSED_THIS_FRAME) {
                printf("[%s] Audio LOST[%d] SID[%d] avIndex[%d]", __func__, frm_no, sid, av_index);
                continue;
            } else if (ret == AV_ER_DATA_NOREADY) {
                usleep(5000);
                continue;
            } else if(ret < 0) {
                printf("[%s] Other error[%d] SID[%d] avIndex[%d]!!!\n", __func__, ret, sid, av_index);
                break;
            } else {
                fps_count++;
                //printf("avRecvAudioData[%d]\n", ret);
            }

            gettimeofday(&tv2, NULL);
            long sec = tv2.tv_sec-tv1.tv_sec, usec = tv2.tv_usec-tv1.tv_usec;
            if(usec < 0) {
                sec--;
                usec += 1000000;
            }
            usec += (sec*1000000);

            if(usec >= 1000000) {
                printf("[Nebula_Device] Recv Spaeker Audio FPS = %d\n", fps_count);
                fps_count = 0;
                gettimeofday(&tv1, NULL);
            }
        }
    }

    printf("[%s] exit, SID[%d] avIndex[%d]\n", __func__, sid, av_index);
    return;
}


//########################################################
//# Handle client command for speaker
//########################################################
int HandleSpeakerControl(int SID, int enable_speaker)
{
    AV_Client *p = &gClientInfo[SID];
    int ret = 0;

    WaitForSingleObject(p->lock, INFINITE);
    p->enable_speaker = enable_speaker;
    ReleaseMutex(p->lock);

    if(1 == enable_speaker) {
        printf("Start Speaker\n");
        int *tmp_sid = (int *)malloc(sizeof(int));
        *tmp_sid = SID;
        HANDLE thread_id = 0;
        thread_id = CreateThread(NULL, 0, ThreadReceiveAudio, (void *)tmp_sid, 0, NULL);
        if (thread_id == 0) {
            printf("pthread_create ThreadReceiveAudio ret[%d]\n", ret);
            return -1;
        } else {
            CloseHandle(thread_id);
        }
    } else {
        printf("Stop Speaker\n");
    }
    return 0;
}

